import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

const settingsOrder = ["sg_hour", "sg_minute", "sg_seconds", "sunOffset"];
const defaultValues = { sg_hour: 0, sg_minute: 0, sg_seconds: 0, sunOffset: 0 };

// Ensure the scoreboard exists and has the default values for settings
if (world.scoreboard.getObjective("settings") == undefined) {
  world.scoreboard.addObjective("settings", "settings");
}

settingsOrder.forEach((setting) => {
  const objective = world.scoreboard.getObjective("settings");
  try {
    objective.getScore(setting);
  } catch (error) {
    // Set the default value if an error occurs
    const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
    objective.setScore(setting, defaultValue);
  }
});

let lastTickTime = Date.now(); // To track the passage of real-world time
let accumulatedTime = 0;
function showTimeForm(player) {
  const hour24 = world.scoreboard.getObjective("settings").getScore("sg_hour");
  const minute = world.scoreboard.getObjective("settings").getScore("sg_minute");
  const sunOffset = world.scoreboard.getObjective("settings").getScore("sunOffset");

  // Convert 24-hour time to 12-hour time
  const hour12 = hour24 % 12 === 0 ? 12 : hour24 % 12; // Convert 0 or 12 to 12
  const isPM = hour24 >= 12;

  const form = new ModalFormData()
    .title("Set Real World Time")
    .slider("§7Use a clock or electronic device to view the current time, and input it here to sync your world up!\n\n§fHour", 1, 12, 1, hour12) // Show 12-hour format
    .slider("Minute", 0, 59, 1, minute)
    .dropdown("AM/PM", ["AM", "PM"], isPM ? 1 : 0) // AM/PM toggle
    .slider("Sun Position Offset §7(-60 to +60 minutes)\n§8(this is for daylight savings or if there is a visual offset with sunset / sunrise times)", -60, 60, 1, sunOffset)
    .submitButton("Save Time");

  form.show(player).then((response) => {
    if (response.isCanceled) {
      showTimeForm(player); // Reopen the form if canceled
    } else if (response.formValues && response.formValues.length >= 4) {
      const selectedHour = response.formValues[0];
      const selectedMinute = response.formValues[1];
      const selectedAMPM = response.formValues[2];
      const selectedSunOffset = response.formValues[3];

      // Convert 12-hour format back to 24-hour format
      let newHour24 = selectedHour;
      if (selectedAMPM === 1 && selectedHour < 12) {
        newHour24 = selectedHour + 12; // Convert PM to 24-hour format
      } else if (selectedAMPM === 0 && selectedHour === 12) {
        newHour24 = 0; // Convert 12 AM to 0 hours (midnight)
      }

      // Store player input in the scoreboard
      world.scoreboard.getObjective("settings").setScore("sg_hour", newHour24);
      world.scoreboard.getObjective("settings").setScore("sg_minute", selectedMinute);
      world.scoreboard.getObjective("settings").setScore("sunOffset", selectedSunOffset);
      world.scoreboard.getObjective("settings").setScore("sg_seconds", 0); // Always reset seconds to 0 when new time is set

      // Apply the initial time of day in the game based on input
      updateGameTime(); // Sync the game time right away
    } else {
      console.log("Invalid form response received.");
    }
  });
}

function showSettings(player) {
  const form = new ModalFormData()
    .title("Personal Settings")
    .toggle("Broadcast Hourly Time on Screen", Boolean(player.hasTag("sg_realistic_timezones:broadcast")))
    .toggle("Display Time on Actionbar", Boolean(player.hasTag("sg_realistic_timezones:display")))
    .toggle("Hourly Bell", Boolean(player.hasTag("sg_realistic_timezones:bell")))
    .submitButton("Save Settings");

  form.show(player).then((response) => {
    if (response.isCanceled) {
      return;
    } else if (response.formValues && response.formValues.length === 3) {
      // Settings are stored in response.formValues
      const broadcastSetting = response.formValues[0];
      const displaySetting = response.formValues[1];
      const bellSetting = response.formValues[2];

      // Handle Broadcast Hourly Time setting
      if (broadcastSetting) {
        if (!player.hasTag("sg_realistic_timezones:broadcast")) {
          player.addTag("sg_realistic_timezones:broadcast");
        }
      } else {
        if (player.hasTag("sg_realistic_timezones:broadcast")) {
          player.removeTag("sg_realistic_timezones:broadcast");
        }
      }

      // Handle Display Time on Actionbar setting
      if (displaySetting) {
        if (!player.hasTag("sg_realistic_timezones:display")) {
          player.addTag("sg_realistic_timezones:display");
        }
      } else {
        if (player.hasTag("sg_realistic_timezones:display")) {
          player.removeTag("sg_realistic_timezones:display");
        }
      }

      // Handle Hourly Bell setting
      if (bellSetting) {
        if (!player.hasTag("sg_realistic_timezones:bell")) {
          player.addTag("sg_realistic_timezones:bell");
        }
      } else {
        if (player.hasTag("sg_realistic_timezones:bell")) {
          player.removeTag("sg_realistic_timezones:bell");
        }
      }
    } else {
      console.log("Invalid form response received.");
    }
  });
}

// Function to calculate and update the game time based on real-world input
function updateGameTime() {
  const hour = world.scoreboard.getObjective("settings").getScore("sg_hour");
  const minute = world.scoreboard.getObjective("settings").getScore("sg_minute");
  const second = world.scoreboard.getObjective("settings").getScore("sg_seconds");
  const sunOffset = world.scoreboard.getObjective("settings").getScore("sunOffset");

  // Total time in minutes including seconds
  const totalMinutes = hour * 60 + minute + second / 60;

  // Calculate the corresponding tick value (0-24000) where 6 AM = tick 0, 6 PM = tick 12000
  let timeOfDay = ((totalMinutes - 6 * 60) * 1000) / 60;

  // Handle negative time values and wrap them around a 24-hour (24000 tick) day
  if (timeOfDay < 0) {
    timeOfDay = 24000 + timeOfDay;
  }

  // Apply sun position offset, converted to ticks
  const offsetTicks = sunOffset * (1000 / 60);
  timeOfDay = (timeOfDay + offsetTicks) % 24000;

  // Set the time of day in the game
  if (Math.round(timeOfDay) === 20) {
    world.getDimension("overworld").runCommand(`time set 20`);
    world.scoreboard.getObjective("settings").addScore("sg_seconds", 5)
    timeOfDay = 21;
  }
  else {
    world.getDimension("overworld").runCommand(`time set ${Math.round(timeOfDay)}`);
  }
}

system.runInterval(() => {
  const hour = world.scoreboard.getObjective("settings").getScore("sg_hour");
  const minute = world.scoreboard.getObjective("settings").getScore("sg_minute");
  const second = world.scoreboard.getObjective("settings").getScore("sg_seconds");
  if (minute === 0 && second === 0) {
    // Broadcast the time to players with the "broadcast" tag
    const playersWithBroadcast = world.getPlayers({
      tags: ["sg_realistic_timezones:broadcast"],
    });
    playersWithBroadcast.forEach((player) => {
      player.runCommand(`titleraw @s subtitle {"rawtext":[{"text":"§e${hour}:00"}]}`);
      player.runCommand(`titleraw @s title {"rawtext":[{"text":"§r"}]}`);
    });

    // Play a bell sound for players with the "bell" tag
    const playersWithBell = world.getPlayers({
      tags: ["sg_realistic_timezones:bell"],
    });
    playersWithBell.forEach((player) => {
      player.runCommand("playsound block.bell.hit @s ~ ~ ~ 1 0.5");
    });
  }
  const playersWithDisplay = world.getPlayers({ tags: ["sg_realistic_timezones:display"] });
  playersWithDisplay.forEach(player => {
    player.runCommand(`titleraw @s actionbar {"rawtext":[{"text":"§e${hour}:${minute < 10 ? "0" + minute : minute}"}]}`);
  });
},20)

function trackAndUpdateTime() {
  const now = Date.now();
  const deltaTime = (now - lastTickTime) / 1000; // Time passed in seconds since last tick
  lastTickTime = now;

  accumulatedTime += deltaTime; // Accumulate time passed

  // Only update seconds when at least one second has passed
  if (accumulatedTime >= 0.1) {
    let currentSeconds = world.scoreboard.getObjective("settings").getScore("sg_seconds");

    // Add full seconds to "sg_seconds" and subtract from accumulated time
    currentSeconds += Math.floor(accumulatedTime);
    accumulatedTime -= Math.floor(accumulatedTime); // Keep leftover fraction of a second

    if (currentSeconds >= 60) {
      currentSeconds -= 60;

      // Update minute and reset seconds if seconds overflow
      let currentMinute = world.scoreboard.getObjective("settings").getScore("sg_minute");
      currentMinute = (currentMinute + 1) % 60;
      world.scoreboard.getObjective("settings").setScore("sg_minute", currentMinute);

      if (currentMinute === 0) {
        // Increment hour if minute overflows
        let currentHour = world.scoreboard.getObjective("settings").getScore("sg_hour");
        currentHour = (currentHour + 1) % 24;
        world.scoreboard.getObjective("settings").setScore("sg_hour", currentHour);
      }
    }

    // Store the updated seconds back to the scoreboard
    world.scoreboard.getObjective("settings").setScore("sg_seconds", currentSeconds);
    
    // Update the game time to reflect new time values
    updateGameTime();
  }
}

// Start the tracking loop on every tick
system.runInterval(trackAndUpdateTime, 1); // 1 tick interval (20 times per second)

// Function to handle initial spawn and display the form
world.afterEvents.playerSpawn.subscribe((event) => {
  if (event.initialSpawn && world.getAllPlayers().length === 1) {
    showTimeForm(event.player);
  }
});




system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_tpa:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_realistic_timezones:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});


world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_realistic_timezones_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_realistic_timezones_version", VERSION);
    if (player.hasTag("sg_realistic_timezones_received_guide")) {
      player.removeTag("sg_realistic_timezones_received_guide");
      player.sendMessage({
        translate: "sg_realistic_timezones.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_realistic_timezones:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_realistic_timezones_version", VERSION);
    player.sendMessage({
      translate: "sg_realistic_timezones.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // UPDATE TIME
    translation_key: "sg_realistic_timezones.guide.options.title",
    icon_path: "textures/ui/timer.png",
    action: (player) => {
      showTimeForm(player);
    },
  },
  {
    // SETTINGS
    translation_key: "sg_realistic_timezones.guide.settings.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {
      showSettings(player);
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_realistic_timezones.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_realistic_timezones.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_realistic_timezones.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_realistic_timezones.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(
        player,
        "sg_realistic_timezones.guide.changelog.title",
        body.build(),
        [
          {
            translation_key: "sg_realistic_timezones.guide.common.back_button",
            action: (player) => showMainPage(player),
          },
        ]
      );
    },
  },
  {
    translation_key: "sg_realistic_timezones.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_realistic_timezones:guide") {
      showMainPage(event.source);
    }
  });
});